% BalticApp
% Model for EE

% Forced choice MNL and MXL/EC
clear
clc

% New data with weights
rng(179424673);
global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c

%% Data transformation 
% cd('C:\Users\budzi\Documents\Praca\Baltic\220108\DATA') % Directory to the folder with data
cd('C:\Users\Admin\Documents\Baltic\DATA')

DATA = load('EE.mat');
EstimOpt.NCT = 1; % Number of choice tasks per person 
EstimOpt.NAlt = 5; % Number of alternatives
EstimOpt.NP = length(DATA.choice)/EstimOpt.NAlt;



DATA.TC = (DATA.tc_km+DATA.tc_time/3);
DATA.EC = [DATA.blue_flag, DATA.comp  == 2, log(DATA.POP3), log(DATA.POP6), log(DATA.POP10)];

Y = reshape(DATA.choice, [EstimOpt.NAlt, EstimOpt.NP]);
VisitInd = sum(Y,1) == 1;
NP2 = sum(VisitInd);
VisitInd = reshape(VisitInd(ones(EstimOpt.NAlt,1),:),[EstimOpt.NAlt*EstimOpt.NP,1]);
EstimOpt.NP = NP2;
Y = DATA.choice(VisitInd);
DATA.TCX = DATA.TC(VisitInd);


DATA.EC = DATA.EC(VisitInd,:);
DATA.W = DATA.rakewt1(VisitInd);

EstimOpt.NamesSites = [DATA.bwid(2:EstimOpt.NAlt); {'Travel Cost (in 100 EUR PPP)'}];


EstimOpt.WTP_space = 0;

Xa = [zeros(1,EstimOpt.NAlt-1);eye(EstimOpt.NAlt-1)];








INPUT.Xa = [repmat(Xa, EstimOpt.NP, 1), -DATA.TCX/100] ;
INPUT.W = reshape(DATA.W, EstimOpt.NAlt, EstimOpt.NP);

INPUT.W = INPUT.W*EstimOpt.NP/sum(INPUT.W(2,:),2);

INPUT.W = reshape(INPUT.W, [EstimOpt.NAlt*EstimOpt.NP,1]);
INPUT.Y = Y;











%% Estimation
cd('C:\Users\Admin\Documents\Baltic\MXL')
EstimOpt.ProjectName = 'EE';
[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NamesA = cellstr(EstimOpt.NamesSites);
%B_backup = [-3.08498478561607;-2.92373716446886;-1.57047947300805;-2.48199254785586;1.90879367046095];
B_backup = [-3.08498485733185;-2.92373391736165;-1.57048091903103;-2.48198670252659;3.81758374832438];
% inny cost (?)
Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% Predicting CS MNL

NP = length(DATA.TC)/EstimOpt.NAlt;
Xa = [repmat(Xa, NP, 1), -DATA.TC/100];

Fit_mnl = reshape(exp(Xa*Results.MNL.bhat),[EstimOpt.NAlt, NP]);
DATA.CS_mnl = log(sum(Fit_mnl)')/Results.MNL.bhat(end);

%% MXL

EstimOpt.Dist = [zeros(1,EstimOpt.NVarA-1)-1,1];
EstimOpt.NumGrad = 0; % (0 - analitical gradient, 1 - numerical gradient)
OptimOpt.GradObj = 'on';
EstimOpt.NRep = 1e3;
EstimOpt.FullCov = 0;
EstimOpt.HessEstFix = 1;

% starting values obtained from MNL
%B_backup = [-3.08498485733185;-2.92373391736165;-1.57048091903103;-2.48198670252659;3.81758374832438;0;0;0;0;1];

% starting values obtained from MXL_d


B_backup = [-3.09212904468488;-2.92932585616598;-1.58194654500270;-2.49881392017219;1.33885590177229;0;0;0;0;-0.119454567055424];
Results.MXL_d.bhat = B_backup;

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);

%% Predicting CS MXL

NSim = 10e4;
cs_sim = NaN(NP,NSim);

mu_lognorm = Results.MXL_d.bhat(end/2);
sigma_lognorm = Results.MXL_d.bhat(end);
% Tutaj NSim x NP

hm1 = sobolset(1,'Skip',EstimOpt.HaltonSkip,'Leap',EstimOpt.HaltonLeap);
hm1 = scramble(hm1,'MatousekAffineOwen');
err_mtx = net(hm1,NSim*NP); % this takes every point:
err_mtx = icdf('Normal',err_mtx,0,1); %to be cut down later
lognorm_random = exp(mu_lognorm+sigma_lognorm*err_mtx);
% lognorm_random = lognrnd(mu_lognorm,sigma_lognorm,[NSim, NP]);
lognorm_random_expanded = reshape(lognorm_random, [1, NSim, NP]);

vec_bhat = Results.MXL_d.bhat(1:EstimOpt.NAlt); %parameters

tic
Fit = reshape(Xa(:,1:end-1)*vec_bhat(1:end-1),[EstimOpt.NAlt, NP]);
Cost = reshape(Xa(:,end),[EstimOpt.NAlt, NP]);
for i = 1:NP
    CostFit = Cost(:,i).*lognorm_random_expanded(:,:,i); % NAlt x NSim
    Fit_i = exp(Fit(:,i) + CostFit); % NAlt x NSim
    esum = max(sum(Fit_i,1), realmin); % to avoid infinite values
    cs_sim(i,:) = log(esum)./lognorm_random_expanded(:,:,i);
    % cs_sim(i,:) = log(sum(Fit_i,1)); % 1 x NSim
end
toc

DATA.CS_mxl_mean = mean(cs_sim,2);
DATA.CS_mxl_median = median(cs_sim,2); % Mozna sprawdzic co wychodzi lepiej w drugim kroku

save 'EE_mxl_cs.mat' DATA Results