% BalticApp
% Model for DE

% Forced choice MNL and MXL/EC  % Trying new dataset from WZ
clear
clc

% New data with weights
rng(179424673);
global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c

%% Data transformation 
% cd('C:\Users\budzi\Documents\Praca\Baltic\220108\DATA') % Directory to the folder with data
cd('C:\Users\Admin\Documents\Baltic\DATA')

load('DE_updated.mat'); % 

%% Data management

% Calculate TC
DATA.HINC(isnan(DATA.HINC)) = median(DATA.HINC(~isnan(DATA.HINC)));
DATA.HHSIZE(isnan(DATA.HHSIZE)) = median(DATA.HHSIZE(~isnan(DATA.HHSIZE)));

% DATA.TCtime = (DATA.TimeR/60).*DATA.HINC./(160*DATA.HHSIZE); %
DATA.TCtime = DATA.TimeR*12.99/60;% 12.99 - using average hourly earning, from TCM 1.21
DATA.TCkm = 0.12543223*DATA.DistanceR; % Przelicznik wziety z TCM_distance data 1.21
DATA.TC = 2*(DATA.TCkm+DATA.TCtime/3);
DATA.Dist = (DATA.DistanceR);

EstimOpt.NAlt = DATA.NAlt; % Number of alternatives
EstimOpt.NP = DATA.NP;
EstimOpt.NCT = 1; % Number of choice tasks per person 

Y = reshape(DATA.Choice, [EstimOpt.NAlt, EstimOpt.NP]);
W = reshape(DATA.Weights_new, [EstimOpt.NAlt, EstimOpt.NP]);
ChosenInd = reshape(DATA.ChosenInd, [EstimOpt.NAlt, EstimOpt.NP]);
TC = reshape(DATA.TC, [EstimOpt.NAlt, EstimOpt.NP]);
Err = reshape(DATA.Err, [EstimOpt.NAlt, EstimOpt.NP]);
Id = reshape(DATA.id, [EstimOpt.NAlt, EstimOpt.NP]);
Dist = reshape(DATA.Dist, [EstimOpt.NAlt, EstimOpt.NP]);

NVis = sum(Y,2);
EstimOpt.NAlt = sum(NVis > 0);
Y = reshape(Y(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);
ChosenInd = reshape(ChosenInd(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);
TC = reshape(TC(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);
Err = reshape(Err(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);
Id = reshape(Id(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);
mDist = reshape(mean(Dist(NVis > 0,:),1), [1, EstimOpt.NP]);
Dist = reshape(Dist(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);
mDist = reshape(mDist(ones(EstimOpt.NAlt,1),:), [EstimOpt.NAlt*EstimOpt.NP,1]);
Id = Id(1:EstimOpt.NAlt:end);
Err = Err(1:EstimOpt.NAlt:end);
W = reshape(W(NVis > 0,:), [EstimOpt.NAlt*EstimOpt.NP,1]);

INPUT.Y = Y(ChosenInd == 2);
DATA.TCX = TC(ChosenInd == 2);
EstimOpt.NP = sum(INPUT.Y);
Sites  = DATA.SiteID(1:38);
Sites = Sites(NVis > 0);
EstimOpt.NamesSites = [Sites(2:end); {'Travel Cost (in 100 EUR PPP)'}];
EstimOpt.WTP_space = 0;

Xa = [zeros(1,EstimOpt.NAlt-1);eye(EstimOpt.NAlt-1)];
INPUT.Xa = [repmat(Xa, EstimOpt.NP, 1), -DATA.TCX/100];

INPUT.W = W(ChosenInd == 2);      
EstimOpt.NVarA = size(INPUT.Xa,2);

%% Estimation
cd('C:\Users\Admin\Documents\Baltic\MXL')
EstimOpt.ProjectName = 'DE';
INPUT.MissingInd = isnan(INPUT.Y);
[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);
% EstimOpt.NumGrad = 1;
EstimOpt.NamesA = cellstr(EstimOpt.NamesSites);

B_backup = [-1.20391400958180;-2.40385856563699;-2.20370159184913;-1.68753279042447;0.908215438871461;0.223336880851126;-1.11479150018516;-2.55226553900655;-1.34318175699477;0.115701134754913;-1.71232205453893;-2.04367720057165;0.284082680324002;1.20716186224833;-0.408675801085386;0.296357167131644;1.91333170313044;0.862331319047999;0.970776048591422;-0.276046946027696;0.127815113124292;-1.55667964570491;-0.590400447877689;-2.58134283363752;-2.98560172110427;-2.61123012631441;-1.12866399916977;-1.71546647317497;-0.893529093401571;3.52085965188355];

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% Predicting CS MNL

NP = length(TC)/EstimOpt.NAlt;
Xa = [repmat(Xa, NP, 1), -TC/100];

Fit_mnl = reshape(exp(Xa*Results.MNL.bhat),[EstimOpt.NAlt, NP]);
DATA.CS = log(sum(Fit_mnl)')/Results.MNL.bhat(end);

%% MXL

EstimOpt.Dist = [zeros(1,EstimOpt.NVarA-1)-1,1];
EstimOpt.NumGrad = 0; % (0 - analitical gradient, 1 - numerical gradient)
OptimOpt.GradObj = 'on';
EstimOpt.NRep = 1e3;
EstimOpt.FullCov = 0;
EstimOpt.HessEstFix = 1;

% starting values obtained from MNL
%B_backup = [-1.20391637390062;-2.40386233156000;-2.20370858774044;-1.68753220367315;0.908215229136280;0.223336526870126;-1.11479267112113;-2.55226650157876;-1.34317957067784;0.115700923317070;-1.71231534426712;-2.04367081002625;0.284084507793165;1.20716147702087;-0.408674249371579;0.296357573900683;1.91333111047782;0.862327241113365;0.970775593693681;-0.276047197089414;0.127815156380996;-1.55668090374157;-0.590400166972288;-2.58133818950451;-2.98560241091407;-2.61123419856515;-1.12866692648757;-1.71546187724777;-0.893525216633722;3.52085837522995;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;1];% starting values obtained from MXL_d

% starting values obtained from MXL_d


B_backup = [-1.20426979819302;-2.40428145574941;-2.20396436010380;-1.68783440754894;0.907972238567744;0.223080238640273;-1.11509674896113;-2.55258510184672;-1.34343833121531;0.115449254183377;-1.71256810434596;-2.04399573911839;0.283799213263288;1.20692693163059;-0.408965727569593;0.296090672839825;1.91309019037653;0.862047960857327;0.970538758504734;-0.276309225222042;0.127525548080394;-1.55698510114652;-0.590635316082604;-2.58170088701527;-2.98597332558126;-2.61161263439855;-1.12898017686858;-1.71571300459904;-0.893787205531361;1.25870770050423;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;0;-0.00583044921308424];
Results.MXL_d.bhat = B_backup;

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);

%% Predicting CS MXL

NSim = 10e4;
cs_sim = NaN(NP,NSim);

mu_lognorm = Results.MXL_d.bhat(end/2);
sigma_lognorm = Results.MXL_d.bhat(end);
% Tutaj NSim x NP

hm1 = sobolset(1,'Skip',EstimOpt.HaltonSkip,'Leap',EstimOpt.HaltonLeap);
hm1 = scramble(hm1,'MatousekAffineOwen');
err_mtx = net(hm1,NSim*NP); % this takes every point:
err_mtx = icdf('Normal',err_mtx,0,1); %to be cut down later
lognorm_random = exp(mu_lognorm+sigma_lognorm*err_mtx);
% lognorm_random = lognrnd(mu_lognorm,sigma_lognorm,[NSim, NP]);
lognorm_random_expanded = reshape(lognorm_random, [1, NSim, NP]);

vec_bhat = Results.MXL_d.bhat(1:EstimOpt.NAlt); %parameters

tic
Fit = reshape(Xa(:,1:end-1)*vec_bhat(1:end-1),[EstimOpt.NAlt, NP]);
Cost = reshape(Xa(:,end),[EstimOpt.NAlt, NP]);
for i = 1:NP
    CostFit = Cost(:,i).*lognorm_random_expanded(:,:,i); % NAlt x NSim
    Fit_i = exp(Fit(:,i) + CostFit); % NAlt x NSim
    esum = max(sum(Fit_i,1), realmin); % to avoid infinite values
    cs_sim(i,:) = log(esum)./lognorm_random_expanded(:,:,i);
    % cs_sim(i,:) = log(sum(Fit_i,1)); % 1 x NSim
end
toc

DATA.CS_mxl_mean = mean(cs_sim,2);
DATA.CS_mxl_median = median(cs_sim,2); % Mozna sprawdzic co wychodzi lepiej w drugim kroku

save 'DE_mxl_cs.mat' DATA Results